--[[
Name: LibTalentQuery-1.0
Revision: $Rev: 64028 $
Author: Rich Martel (richmartel@gmail.com)
Documentation: http://wowace.com/wiki/LibTalentQuery-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibTalentQuery-1.0
Description: Library to help with querying unit talents
Dependancies: LibStub, CallbackHandler-1.0
License: LGPL v2.1

Example Usage:
	local TalentQuery = LibStub:GetLibrary("LibTalentQuery-1.0")
	TalentQuery.RegisterCallback(self, "TalentQuery_Ready")
	TalentQuery:Query(unit)
	local spec = {}
	function MyAddon:TalentQuery_Ready(e, unit)
		for tab = 1, GetNumTalentTabs(true) do
			local treename, _, pointsspent = GetTalentTabInfo(tab, true)
			if (pointsspent > 30) then
				spec[unit] = treename
			end
		end
		...
	end
]]

local INSPECTDELAY = 2
local MAJOR, MINOR = "LibTalentQuery-1.0", "$Rev: 64028 $"

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.events = lib.events or LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = lib.frame or CreateFrame("Frame", MAJOR .. "_Frame")
lib.frame:SetScript("OnEvent", function(this, event, ...) lib[event](lib, ...) end)
lib.frame:SetScript("OnUpdate", function() return lib:OnUpdate() end)

local inspectingUnit = nil
local inspectQueue = {}
local nextInspect = nil

hooksecurefunc("NotifyInspect", function(...) return lib:NotifyInspect(...) end)
function lib:NotifyInspect(unit)
	-- If this library did not call NotifyInspect, and we are waiting for a query, then
	-- stop waiting, and put the inspectingUnit back on the queue.
	if inspectingUnit and (inspectingUnit ~= unit) then
		self.frame:UnregisterEvent("INSPECT_TALENT_READY")
		inspectQueue[inspectingUnit] = true
		inspectingUnit = nil
		nextInspect = GetTime() + INSPECTDELAY
	end
end

function lib:INSPECT_TALENT_READY()
	local unit
	if inspectingUnit then
		self.frame:UnregisterEvent("INSPECT_TALENT_READY")
		unit = inspectingUnit
		inspectingUnit = nil
	end
	self.events:Fire("TalentQuery_Ready", unit)
end

function lib:CheckInspectQueue()
	local repeatqueuecheck = nil
	if (not inspectingUnit) then
		for unit in pairs(inspectQueue) do
			repeatqueuecheck = true
			if UnitIsVisible(unit) and UnitIsConnected(unit) then
				inspectingUnit = unit
				inspectQueue[inspectingUnit] = nil
				self.frame:RegisterEvent("INSPECT_TALENT_READY")
				NotifyInspect(inspectingUnit)
				break
			end
		end
	else
		repeatqueuecheck = true
	end
	if repeatqueuecheck then
		nextInspect = GetTime() + INSPECTDELAY
	else
		nextInspect = nil
	end
end

function lib:OnUpdate()
	if nextInspect and (nextInspect < GetTime()) then
		self:CheckInspectQueue()
	end
end

function lib:Query(unit)
	inspectQueue[unit] = true
	nextInspect = GetTime() + INSPECTDELAY
end
